s/@@NASM_MAJOR@@/2/g
s/@@NASM_MINOR@@/14/g
s/@@NASM_SUBMINOR@@/2/g
s/@@NASM_PATCHLEVEL@@/0/g
s/@@NASM_SNAPSHOT@@/0/g
s/@@NASM_VERSION_ID@@/34472448/g
s/@@NASM_VERSION_XID@@/0x020e0200/g
s/@@NASM_VER@@/2.14.02/g
s/@@NASM_MANGLED_VER@@/2.14.02/g
